#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   tagHelper.py
@Time    :   2019/07/18
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :   
'''
import os
import sys
from mutagen import File
from mutagen import flac
from mutagen import mp4
from mutagen.id3 import TALB, TCOP, TDRC, TIT2, TPE1, TRCK, APIC, TCON, TCOM
from aigpy import pathHelper

def _getHash(pHash, key):
    if key in pHash:
        return pHash[key]
    return ''

def _lower(inputs):
    if isinstance(inputs, str):
        inputs = inputs.decode('utf-8')
    inputs = inputs.lower().encode('utf-8')
    return inputs

def _getExtension(filepath):
    index = filepath.rfind('.')
    ret = filepath[index + 1:len(filepath)]
    v = sys.version_info
    if v[0] > 2:
        return str.lower(ret)
    else:
        return _lower(ret)

def _getFileData(filepath):
    try:
        with open(filepath, "rb") as f:
            data = f.read()
            return data
    except:
        return None

def _tryInt(obj):
    try:
        ret = int(obj)
        return ret
    except:
        return 0

def _getArrayStr(array):
    if array is None:
        return ''
    if len(array) <= 0:
        return array
    ret = None
    for item in array:
        if ret is None:
            ret = item
            continue
        ret += ';' + item
    return ret

def _noneToEmptyString(obj):
    if obj is None:
        return ''
    else:
        return obj

class TagTool(object):
    def __init__(self, filePath):
        if os.path.isfile(filePath) is False:
            return

        self._filepath = filePath
        self._ext = _getExtension(filePath)
        self._handle = File(filePath)

        self.title = ''
        self.album = ''
        self.albumartist = ''
        self.artist = ''
        self.copyright = ''
        self.tracknumber = ''
        self.totaltrack = ''
        self.discnumber = ''
        self.totaldisc = ''
        self.genre = ''
        self.date = ''
        self.composer = ''

    def save(self, coverPath=None):
        try:
            if 'mp3' in self._ext:
                return self._saveMp3(coverPath)
            if 'flac' in self._ext:
                return self._saveFlac(coverPath)
            if 'mp4' in self._ext or 'm4a' in self._ext:
                return self._saveMp4(coverPath)
            return False
        except:
            return False

    def _saveMp3(self, coverPath):
        self._handle.tags.add(TIT2(encoding=3, text=self.title))
        self._handle.tags.add(TALB(encoding=3, text=self.album))
        # self._handle.tags.add(TOPE(encoding=3, text=self.albumartist))
        self._handle.tags.add(TPE1(encoding=3, text=self.artist))
        self._handle.tags.add(TCOP(encoding=3, text=self.copyright))
        self._handle.tags.add(TRCK(encoding=3, text=str(self.tracknumber)))
        # self._handle.tags.add(TRCK(encoding=3, text=self.discnum))
        self._handle.tags.add(TCON(encoding=3, text=self.genre))
        self._handle.tags.add(TDRC(encoding=3, text=self.date))
        self._handle.tags.add(TCOM(encoding=3, text=self.composer))
        self._savePic(coverPath)
        self._handle.save()
        return True

    def _saveFlac(self, coverPath):
        if self._handle.tags is None:
            self._handle.add_tags()
        self._handle.tags['title'] = self.title
        self._handle.tags['album'] = self.album
        self._handle.tags['albumartist'] = self.albumartist
        self._handle.tags['artist'] = self.artist
        self._handle.tags['copyright'] = _noneToEmptyString(self.copyright)
        self._handle.tags['tracknumber'] = str(self.tracknumber)
        self._handle.tags['tracktotal'] = str(self.totaltrack)
        self._handle.tags['discnumber'] = str(self.discnumber)
        self._handle.tags['disctotal'] = str(self.totaldisc)
        self._handle.tags['genre'] = _noneToEmptyString(self.genre)
        self._handle.tags['date'] = _noneToEmptyString(self.date)
        self._handle.tags['composer'] = _noneToEmptyString(self.composer)
        self._savePic(coverPath)
        self._handle.save()
        return True
    
    def _saveMp4(self, coverPath):
        self._handle.tags['\xa9nam'] = self.title
        self._handle.tags['\xa9alb'] = self.album
        self._handle.tags['aART'] = _getArrayStr(self.albumartist)
        self._handle.tags['\xa9ART'] = _getArrayStr(self.artist)
        self._handle.tags['cprt'] = _noneToEmptyString(self.copyright)
        self._handle.tags['trkn'] = [[_tryInt(self.tracknumber), _tryInt(self.totaltrack)]]
        self._handle.tags['disk'] = [[_tryInt(self.discnumber), _tryInt(self.totaldisc)]]
        self._handle.tags['\xa9gen'] = _noneToEmptyString(self.genre)
        self._handle.tags['\xa9day'] = _noneToEmptyString(self.date)
        self._handle.tags['\xa9wrt'] = _getArrayStr(self.composer)
        self._savePic(coverPath)
        self._handle.save()
        return True

    def _savePic(self, coverPath):
        data = _getFileData(coverPath)
        if data is None:
            return
        if 'flac' in self._ext:    
            pic = flac.Picture()
            pic.data = data
            if pathHelper.getFileExtension(coverPath) == '.jpg':
                pic.mime = u"image/jpeg"
            self._handle.clear_pictures()
            self._handle.add_picture(pic)
        if 'mp3' in self._ext:
            self._handle.tags.add(APIC(encoding=3, data=data))
        if 'mp4' in self._ext or 'm4a' in self._ext:
            pic = mp4.MP4Cover(data)
            self._handle.tags['covr'] = [pic]

# test = TagTool('e:\\1.m4a')
# test.album = ['ff']
# test.albumartist = ['yaron', 'f']
# test.artist = ['huang', 'dd']
# test.copyright = None
# test.title = 'yes'
# test.tracknumber = '1'
# test.genre = 'fdsa'
# test.date = '2019-2-3'
# test.save('e:\\1.jpg')
# y =str.lower("Ye")

# t = str("Ye").lower()
# a = o



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#coding:utf-8
'''
@File    :   ffmpegHelper.py
@Time    :   2018/12/17
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :   
'''

import os
import re
import sys
import shutil
import subprocess
import aigpy.fileHelper as fileHelper
import aigpy.systemHelper as systemHelper
import aigpy.netHelper as netHelper
import aigpy.pathHelper as pathHelper
import aigpy.threadHelper as threadHelper
from   aigpy.progressHelper import ProgressTool

class FFmpegTool(object):
    def __init__(self, threadNum=50, mergerTimeout=None):
        """
        #Func    :   初始化             
        #Param   :   threadNum     [in] 线程数          
        #Param   :   mergerTimeout [in] 超时 秒     
        #Return  :   True/False             
        """
        self.thread        = threadHelper.ThreadTool(threadNum)
        self.waitCount     = 0
        self.completeCount = 0
        self.progress      = None
        self.mergerTimeout = mergerTimeout
        self.enable        = self._checkTool()
        return
    
    def __thradfunc_dl(self, url, filepath, retrycount):
        check = False
        try:
            while retrycount > 0:
                retrycount = retrycount - 1
                check = netHelper.downloadFile(url, filepath, 30)
                if check:
                    break
        except:
            pass
        self.completeCount = self.completeCount + 1
        if self.progress is not None:
            self.progress.step()
        return
    
    def __parseM3u8(self, url):
        content = netHelper.downloadString(url, None)
        pattern = re.compile(r"(?<=http).+?(?=\\n)")
        plist   = pattern.findall(str(content))
        urllist = []
        for item in plist:
            urllist.append("http"+item)
        return urllist

    def __process(self, cmd, retrycount, showshell, filename, removeFile=True):
        stdoutFile = None
        fp = None
        while retrycount >= 0:
            retrycount -= 1
            try:
                if showshell:
                    if sys.version_info[0] > 2:
                        res = subprocess.call(cmd, timeout=self.mergerTimeout, shell=True)
                    else:
                        cmd = cmd.encode(sys.getfilesystemencoding())
                        res = subprocess.call(cmd, shell=True)
                else:
                    exten = pathHelper.getFileExtension(filename)
                    stdoutFile = filename.replace(exten, '-stdout.txt')
                    fp  = open(stdoutFile, 'w')
                    if sys.version_info[0] > 2:
                        res = subprocess.call(cmd, timeout=self.mergerTimeout, shell=True, stdout=fp, stderr=fp)
                    else:
                        res = subprocess.call(cmd, shell=True, stdout=fp, stderr=fp)
                    fp.close()
                    fp = None
                    pathHelper.remove(stdoutFile)
                if res == 0:
                    return True
            except:
                pass
            if fp:
                fp.close()
            pathHelper.remove(stdoutFile)
            if removeFile: 
                pathHelper.remove(filename)
        return False

    def mergerByM3u8_Multithreading2(self, url, filepath, showprogress=False, showshell=False):
        try:
            # Get urllist
            urllist = self.__parseM3u8(url)
            if len(urllist) <= 0:
                return False

            ext    = pathHelper.getFileExtension(filepath)
            tspath = filepath.replace(ext, '.ts')
            pathHelper.remove(tspath)
            if not netHelper.downloadFileByUrls(urllist, tspath, 30, True):
                return False
            if self.covertFile(tspath, filepath):
                pathHelper.remove(tspath)
                return True
            return False
        except:
            return False

    def mergerByM3u8_Multithreading(self, url, filepath, showprogress=False, showshell=False):
        """
        #Func    :   多线程下载并合并文件(使用M3u8的url)        
        #Param   :   url             [in] 链接
        #Param   :   filepath        [in] 目标文件名
        #Param   :   showprogress    [in] 是否显示进度条
        #Param   :   showshell       [in] 是否显示cmd信息
        #Return  :   True/False
        """
        try:
            # Get urllist
            urllist = self.__parseM3u8(url)
            if len(urllist) <= 0:
                return False

            # Creat tmpdir
            path    = pathHelper.getDirName(filepath)
            tmpPath = pathHelper.getDiffTmpPathName(path)
            if pathHelper.mkdirs(tmpPath) is False:
                return False
            
            # Progress
            self.progress = None
            if showprogress:
                self.progress = ProgressTool(len(urllist))

            # Download files
            allpath            = []
            self.waitCount     = len(urllist)
            self.completeCount = 0
            for i, item in enumerate(urllist):
                index = i + 100001
                path  = tmpPath + '/' + str(index) + ".ts"
                path  = os.path.abspath(path)
                allpath.append(path)
                if os.path.exists(path):
                    os.remove(path)
                self.thread.start(self.__thradfunc_dl, item, path, 3)
            self.thread.waitAll()
            ret = self.mergerByTs(tmpPath, filepath, showshell)
            # ret = self.mergerByFiles(allpath, filepath, showshell)
            shutil.rmtree(tmpPath)
            return ret
        except:
            return False

    def mergerByM3u8(self, url, filepath, showshell=False):
        """
        #Func    :   合并文件(使用M3u8的url)        
        #Param   :   url         [in] 链接       
        #Param   :   filepath    [in] 目标文件名            
        #Param   :   showshell   [in] 是否显示cmd信息              
        #Return  :   True/False 
        """
        res = -1
        try:
            filepath = os.path.abspath(filepath)
            cmd = "ffmpeg -safe 0 -i " + url + " -c copy -bsf:a aac_adtstoasc \"" + filepath + "\""
            res = self.__process(cmd, 3, showshell, filepath)
        except:
            pass
        return res

    def mergerByTs(self, srcDir, filepath, showshell=False):
        srcDir   = os.path.abspath(srcDir)
        filepath = os.path.abspath(filepath)
        if os.path.exists(srcDir) is False:
            return False
        
        exten   = pathHelper.getFileExtension(filepath)
        tmppath = filepath.replace(exten, '.ts')
        if systemHelper.isWindows():
            srcDir += '\\*.ts'
            cmd = 'copy /b "' + srcDir + '" "' + tmppath + '"'
        else:
            srcDir += '/*.ts'
            cmd = 'cat ' + srcDir + ' > "' + tmppath + '"'
        
        ret = self.__process(cmd, 3, showshell, tmppath)
        if ret is True:
            cmd = "ffmpeg -i \"" + tmppath + "\" -c copy \"" + filepath + "\""
            ret = self.__process(cmd, 3, showshell, filepath)
        pathHelper.remove(tmppath)
        return ret


    def mergerByTsfiles(self, srcfilepaths, filepath, showshell=False):
        """
        #Func    :   合并ts文件             
        #Return  :   True/False         
        """
        filepath = os.path.abspath(filepath)
        exten    = pathHelper.getFileExtension(filepath)
        tmppath  = filepath.replace(exten, '.ts')
        tmppath2 = filepath.replace(exten, '2.ts')
        array    = [srcfilepaths[i:i+25] for i in range(0, len(srcfilepaths), 25)]
        pathHelper.remove(tmppath)
        pathHelper.remove(tmppath2)

        for item in array:
            for index, file in enumerate(item):
                item[index] = '"'+ file + '"'
            form = ' + '.join(item)
            if os.access(tmppath, 0):
                form = '"' + tmppath + '" + ' + form
            
            cmd = 'copy /b ' + form + ' "' + tmppath2 + '"'
            ret = self.__process(cmd, 3, showshell, tmppath2)
            if ret is False:
                break
            pathHelper.remove(tmppath)
            os.rename(tmppath2, tmppath)

        if ret is True:
            cmd = "ffmpeg -i \"" + tmppath + "\" -c copy \"" + filepath + "\""
            ret = self.__process(cmd, 3, showshell, filepath)
        pathHelper.remove(tmppath)
        pathHelper.remove(tmppath2)
        return ret

    def _checkTool(self):
        check = False
        try:
            cmd = "ffmpeg -V"
            stdoutFile = 'ffmpegcheck-stdout.txt'
            fp = open(stdoutFile, 'w')
            if sys.version_info[0] > 2:
                res = subprocess.call(cmd, timeout=self.mergerTimeout, shell=True, stdout=fp, stderr=fp)
            else:
                res = subprocess.call(cmd, shell=True, stdout=fp, stderr=fp)
            fp.close()
            txt = fileHelper.getFileContent(stdoutFile)
            if 'version' in txt and 'Copyright' in txt:
                check = True
        except:
            pass
        pathHelper.remove(stdoutFile)
        return check
    def covertFile(self, srcfile, descfile, showshell=False):
        try:
            filepath = os.path.abspath(srcfile)
            filepath2 = os.path.abspath(descfile)
            cmd = "ffmpeg -i \"" + filepath + "\" -c copy \"" + filepath2 + "\""
            ret = self.__process(cmd, 3, showshell, filepath, False)
            return ret
        except:
            return False

    def mergerByFiles(self, srcfilepaths, filepath, showshell=False):
        """
        #Func    :   合并文件       
        #Param   :   srcfilepaths   [in] 文件名数组     
        #Param   :   filepath       [in] 目标文件名     
        #Param   :   showshell      [in] 是否显示cmd信息        
        #Return  :   True/False 
        """
        filepath = os.path.abspath(filepath)
        res     = -1
        tmpfile = filepath + "TMP.txt"
        paths   = srcfilepaths
        group   = None
        groupnum= 50
        try:
            # 先分组合并，再一起合并
            if len(srcfilepaths) > groupnum:
                dirName = pathHelper.getDirName(filepath)
                group   = pathHelper.getDiffTmpPathName(dirName)
                if pathHelper.mkdirs(group) is False:
                    return False
                
                newPaths = []
                array    = [srcfilepaths[i:i+groupnum] for i in range(0, len(srcfilepaths), groupnum)]
                for index, item in enumerate(array):
                    file = group + '/' + str(index) + ".mp4"
                    newPaths.append(file)
                    if self.mergerByFiles(item, file, showshell) is False:
                        return False
                paths = newPaths

            # 创建临时文件,将要合并的文件名列表写入
            with open(tmpfile, 'w') as fd:
                for item in paths:
                    item = os.path.abspath(item)
                    fd.write('file \'' + item + '\'\n')


            # 调用ffmpeg进行合并
            cmd = "ffmpeg -f concat -safe 0 -i \"" + tmpfile + "\" -c copy \"" + filepath + "\""
            res = self.__process(cmd, 3, showshell, filepath)
        except:
            pass

        if os.access(tmpfile,0):
            os.remove(tmpfile)
        if group is not None:
            shutil.rmtree(group)
        return res

    def test(self):
        self.__process('dir', 3, False, 'e:\\7\\Video\\1.ts')


# tool = FFmpegTool(1)
# src = u'e:\\7\\Album\\Beyonc\xe9\\Lemonade/01 Pray You Catch Me.mp4'
# # s = src.encode(sys.getfilesystemencoding())
# # short_path = win32api.GetShortPathName(src)
# desc = u'e:\\7\\Album\\Beyonc\xe9\\Lemonade/01 Pray You Catch Me.m4a'
# tool.covertFile(src, desc)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   zipHelper.py
@Time    :   2019/03/01
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :   Zip Tool
'''

import os
import zipfile
import tarfile

def _getParaType(para):
    try:
        if os.path.isfile(para):
            return 0
    except:
        pass
    try:
        if os.path.isdir(para):
            return 1
    except:
        pass
    return 2

def _getZipType(zipName):
    try:
        name = os.path.basename(zipName)
        if name.lower().find('.tar') > 0:
            return 'tar'
        else:
            return 'zip'
    except:
        pass

def _open(zipName, ptype, mode='w'):
    try:
        if ptype == 'tar':
            pZip = tarfile.open(zipName, mode)
        else:
            pZip = zipfile.ZipFile(zipName, mode, zipfile.ZIP_DEFLATED)
        return pZip
    except:
        return None

def _write(pZip, ptype, pfilename, parcname):
    try:
        if ptype == 'tar':
            pZip.add(pfilename, arcname=parcname)
        else:
            pZip.write(pfilename, arcname=parcname)
        return True
    except:
        return False


def myzip(para, zipName):
    """
    #Func    :   zip files or dir       
    #Param   :   para      file | file[] | dir      
    #Param   :   zipName   outPathName      
    #Return  :   True/False         
    """
    check = _getParaType(para)
    ptype = _getZipType(zipName)
    try: 
        pZip = _open(zipName, ptype)
        if check == 2:
            for file in para:
                pZip._write(pZip, ptype, file, os.path.basename(file))
        if check == 0:
            pZip._write(pZip, ptype, para, os.path.basename(para))
        if check == 1:
            name = os.path.dirname(para)
            for dirpath, dirnames, filenames in os.walk(para):
                fpath = dirpath.replace(name, '')
                fpath = fpath and fpath + os.sep or ''
                for filename in filenames:
                    pZip._write(pZip, ptype, os.path.join(dirpath, filename), fpath +filename)
        pZip.close()
        return True
    except:
        return False

def myunzip(zipName, outPath):
    ptype = _getZipType(zipName)
    try:
        pZip = _open(zipName, ptype, 'r')
        pZip.extractall(path=outPath)
        pZip.close()
        return True
    except:
        return False



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   logHelper.py
@Time    :   2019/02/28
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :   LOG FILE TOOL
'''
import os
import time

def write(path, string):
    try:
        fd = open(path, 'a+')
        fd.write(string + '\n')
        fd.close()
        return True
    except:
        return False  

def writeByTime(path, string):
    date = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))
    return write(path, '[' + date + ']  ' + string)
 
 
def clear(path):
    try:
        os.remove(path)
    except:
        pass



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   stringHelper.py
@Time    :   2019/03/11
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :   
'''



def isChinese(word, checkPunctuation = False):
    punctuationStr = '，。！？【】（）％＃＠＆１２３４５６７８９０：'
    for ch in word:
        if '\u4e00' <= ch <= '\u9fff':
            return True
        if checkPunctuation and punctuationStr.find(ch) != -1:
            return True
    return False


def converPunctuationToEnglish(word):
    table = {ord(f): ord(t) for f, t in zip(
        u'，。！？【】（）％＃＠＆１２３４５６７８９０：',
        u',.!?[]()%#@&1234567890:')}
    ret = word.translate(table)
    return ret


def align(string, num, isLeft=True):
    leng = 0
    for c in string:
        leng += 1
        if isChinese(c, True):
            leng += 1
    if leng >= num:
        return string
    appendStr = ""
    while num - leng > 0:
        appendStr += " "
        num -= 1
    if isLeft:
        return string + appendStr
    else:
        return appendStr + string




#!/usr/bin/env python
# -*- encoding: utf-8 -*-

'''
@File    :   serverHelper.py
@Time    :   2019/04/10
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :   
'''

# -*- coding=utf-8 -*-
import socket

from aigpy.threadHelper import ThreadTool
from aigpy.threadHelper import ThreadPoolManger
from aigpy.httpHelper   import HttpRequest

class ServerTool(object):
    def __init__(self, rootDir,cookieDir,fileOf404=None,scfamily=socket.AF_INET,sctype=socket.SOCK_STREAM):
        """
        #Func    :   初始化     
        #Param   :   rootDir    [in] 根目录     
        #Param   :   cookieDir  [in] cookie目录     
        #Param   :   fileOf404  [in] 404文件        
        #Param   :   scfamily   [in] 网络family     
        #Param   :   sctype     [in] 网络类型       
        """
        rootDir.replace('\\', '/')
        cookieDir.replace('\\', '/')
        self.__sockHandle    = None
        self.__sockFamily    = scfamily
        self.__sockType      = sctype
        self.__rootDir       = rootDir + '/'
        self.__cookieDir     = cookieDir + '/'
        self.__fileOf404     = fileOf404
        self.__requestFunc   = None
        self.__listenThread  = ThreadTool(1)
        self.__requestThread = ThreadPoolManger(5)
        self.__revieveLen    = 1024

    def __requestThreadCall__(self, sock, addr):
        body     = None
        request  = sock.recv(self.__revieveLen)
        http_req = HttpRequest(self.__rootDir, self.__cookieDir, self.__fileOf404)
        http_req.passRequest(request)

        if self.__requestFunc is not None:
            body = self.__requestFunc(http_req)
        sock.send(http_req.getResponse(body).encode('utf-8'))
        sock.close()

    def __listenThreadCall__(self):
        while True:
            sock, addr = self.__sockHandle.accept()
            self.__requestThread.addWork(self.__requestThreadCall__, *(sock, addr))
        self.start()
    def start(self, address, port, requestFuc=None, listenNum=10, recieveLen=1024):
        """
        #Func    :   启动
        #Param   :   address    [in] ip地址 
        #Param   :   port       [in] 端口号
        #Param   :   requestFuc [in] 响应函数,参数为httpRequest,返回body     
        #Param   :   listenNum  [in] 监听的数量
        #Param   :   recieveLen [in] 数据包的长度
        #Return  :   True/False   
        """
        try:
            self.stop()
            self.__revieveLen  = recieveLen
            self.__requestFunc = requestFuc
            self.__sockHandle  = socket.socket(self.__sockFamily, self.__sockType)
            self.__sockHandle.bind((address, int(port)))
            self.__sockHandle.listen(listenNum)
            self.__listenThread.start(self.__listenThreadCall__)
            return True
        except:
            return False

    def stop(self):
        if self.__sockHandle is None:
            return
        self.__listenThread.close()
        self.__requestThread.close()
        self.__sockHandle.close()







#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   pipHelper.py
@Time    :   2019/03/11
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :   pip server tool
'''

import aigpy.netHelper as netHelper

def getInfo(projectName):
    """Get project information from pypi
    - Return: json or None                              
    """
    url = 'https://pypi.org/pypi/' + projectName + '/json'
    ret = netHelper.downloadJson(url,None)
    return ret
    
def getLastVersion(projectName):
    """Get project version from pypi
    - Return: str or None                              
    """
    try:
        ret = getInfo(projectName)
        if ret is None:
            return None
        return ret['info']['version']
    except:
        return None

def getVersionList(projectName):
    """Get project all versions from pypi
    - Return: json or None                              
    """
    try:
        ret = getInfo(projectName)
        if ret is None:
            return None
        return ret['releases']
    except:
        return None



# -*- coding: utf-8 -*-

def convertStorageUnit(num, srcUnit, desUnit):
    """Convert unit

    - num: value
    - srcUnit: gb/mb/kb/byte
    - desUnit: gb/mb/kb/byte
    """
    try:
        units = ['gb','mb','kb','byte']
        if srcUnit not in units or desUnit not in units:
            return None
        if srcUnit == desUnit:
            return num
        num = float(num)
        if num == 0:
            return 0
        srcIndex = units.index(srcUnit)
        desIndex = units.index(desUnit)
        tmp = desIndex - srcIndex
        while tmp != 0:
            if srcIndex < desIndex:
                num = num * 1024
            else:
                num = num / 1024
            if tmp > 0:
                tmp = tmp - 1
            else:
                tmp = tmp + 1
        return num
    except:
        return None

def convertStorageUnitToString(num, srcUnit):
    """Convert unit to string
    
    - num: value
    - srcUnit: gb/mb/kb/byte
    """
    try:
        units = ['gb', 'mb', 'kb', 'byte']
        if srcUnit not in units:
            return '0 KB'
        num = float(num)
        srcIndex = units.index(srcUnit)
        if srcIndex == 0:
            return str(round(num, 2)) + ' ' + units[0].upper()
        
        tmp = num
        while srcIndex != 0:
            num = num / 1024
            if num < 1:
                return str(round(tmp, 2)) + ' ' + units[srcIndex].upper()
            tmp = num
            srcIndex = srcIndex - 1
        return str(round(tmp, 2)) + ' ' + units[srcIndex].upper()
    except:
        return '0 KB'



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

'''
@File    :   systemHelper.py
@Time    :   2018/12/20
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :   
'''
import os
import platform
import subprocess
import requests


def getOwnPath(in__file__):
    return os.path.dirname(os.path.realpath(in__file__))

def isWindows():
    sysName = platform.system()
    return sysName == "Windows"

def isLinux():
    sysName = platform.system()
    return sysName == "Linux"

def getProcessID(name):
    try:
        retid = []
        if isLinux():
            lines = os.popen('ps aux | grep "' + name + '" | grep -v grep').readlines()
            if len(lines) <= 0:
                return []
            for item in lines:
                array = item.split()
                retid.append(int(array[1]))
        else:
            import psutil
            pidList = list(psutil.process_iter())
            for item in pidList:
                # 样式为："psutil.Process(pid=0, name='System Idle Process', started='2019-04-01 08:38:07')"
                stri = str(item)
                stri = stri[15:-1]
                # 取名
                itname = stri[stri.find("name")+6:-1]
                indx = itname.find(",")
                if indx >= 0:
                    itname = itname[0:indx-1]
                if itname != name:
                    continue
                # 取pid
                pid = stri[stri.find("pid")+4:stri.find("name")-2]
                retid.append(int(pid))
        return retid
    except:
        return []


def killProcess(proid):
    try:
        if isLinux():
            os.popen('kill -9 ' + str(proid))
            lines = os.popen('ps ' + str(proid)).readlines()
            if len(lines) <= 1:
                return True
        else:
            lines = os.popen('tasklist | findstr ' + str(proid)).readlines()
            if len(lines) <= 0:
                return True
            unread2 = os.popen('taskkill /pid %s /f' % str(proid))
        return False
    except:
        return False

def openPort(port):
    cmds = [
        "iptables -I INPUT -m state --state NEW -m tcp -p tcp --dport " + str(port) + " -j ACCEPT",
        "iptables -I INPUT -m state --state NEW -m udp -p udp --dport " + str(port) + " -j ACCEPT",
        "ip6tables -I INPUT -m state --state NEW -m tcp -p tcp --dport " + str(port) + " -j ACCEPT",
        "ip6tables -I INPUT -m state --state NEW -m udp -p udp --dport " + str(port) + " -j ACCEPT"
    ]
    for item in cmds:
        subprocess.call(item, shell=False)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   m3u8Helper.py
@Time    :   2019/08/23
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :   
'''
import re
from aigpy import netHelper

def getM3u8TsUrls(url):
    content = netHelper.downloadString(url, None)
    pattern = re.compile(r"(?<=http).+?(?=\\n)")
    plist   = pattern.findall(str(content))
    urllist = []
    for item in plist:
        urllist.append("http"+item)
    return urllist




# url = 'http://api.tidal.com/v1/videos/92418079/hls/CAEQARgDIP___________wEouA8yQzgyNjFkMWZmLTYwNmEtNDNkNy04YzFjLTA0YzVhNzI3YzllZV8xMjgvbWVkaWFfcGxheWxpc3RfMjUwNTIxLm0zdTg=.m3u8?authtoken=exp~1566527826000.hmac~U5uKekcIxE_cqhGcp9HJ70kYV3d7qWQUv8FTQiSezz0='
# getM3u8TsUrls(url)


#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   configHelper.py
@Time    :   2018/12/17
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :   Config Tool
'''
import os
import configparser

def Count(fileName, section=None):
    """Get para number"""
    try:
        ret = 0
        cf = configparser.ConfigParser()
        cf.read(fileName)
        if section is None:
            seclist = cf.sections()
            for sec in seclist:
                oplist = cf.options(sec)
                ret    = ret + len(oplist)
        elif cf.has_section(section):
            ret = len(cf[section])
        return ret
    except:
        return 0

def Sections(fileName):
    """Get groups"""
    try:
        cf = configparser.ConfigParser()
        cf.read(fileName)
        return cf.sections()
    except:
        return None


def GetValue(section, key, default, fileName):
    try:
        cf = configparser.ConfigParser()
        cf.read(fileName)
        if not cf.has_section(section):
            return default
        
        if key in cf[section]:
            default = cf.get(section, key)
        return default
    except:
        return default

def SetValue(section, key, value, fileName):
    try:
        if os.access(fileName, 0) is False:
            fp = open(fileName, "w")
            fp.close()

        cf = configparser.ConfigParser()
        cf.read(fileName)
        if cf.has_section(section) is False:
            cf[section] = {}

        cf[section][key] = value
        with open(fileName, "w") as f:
            cf.write(f)
        return True
    except:
        return False


def ParseNoEqual(fileName):
    ret = {}
    try:
        fd  = open(fileName, 'r')
        arr = fd.readlines()
        group = None
        for item in arr:
            item = item.strip()
            if len(item) <= 0:
                continue
            if item[0] == '#':
                continue
            elif item[0] == '[' and item[len(item) - 1] == ']':
                group = item[1:len(item) - 1]
                ret[group] = []
            elif group is None:
                continue
            else:
                ret[group].append(item)
        return ret     
    except:
        return ret




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   cmdHelper.py
@Time    :   2019/02/27
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :   
'''

import sys
from enum import Enum
from colorama import init

init(autoreset=True)

def isInputYes(inputstr):
    """Return: bool"""
    if inputstr is None:
        return False
    inputstr = str(inputstr).lower()
    if inputstr == 'yes' or inputstr == 'y':
        return True
    return False


def myinput(desc):
    if sys.version_info[0] > 2:
        return input(desc)
    else:
        ret = raw_input(desc)
        if len(ret) > 0:
            if '\r' == ret[len(ret)-1:]:
                ret = ret[:len(ret)-1]
        return ret


def myinputInt(desc, default):
    try:
        stri = myinput(desc)
        ret = int(stri)
        return ret
    except:
        return default


def myinputFloat(desc, default):
    try:
        stri = myinput(desc)
        ret  = float(stri)
        return ret
    except:
        return default


def myprintNoEnter(desc):
    sys.stdout.write(desc)


def findInArgv(stri):
    if sys.argv is None or len(sys.argv) == 0:
        return None
    
    for item in sys.argv:
        if item == sys.argv[0]:
            continue
        if item.find(stri) >= 0:
            return item
    return None


def converArgvToStr(array):
    stri = ''
    for item in array:
        if stri != '':
            stri = stri + ' '
        stri = stri + '"' + item + '"'
    return stri


class TextColor(Enum):
    """Font color"""
    Black  = 30
    Blue   = 34
    Green  = 32
    Red    = 31
    Yellow = 33
    White  = 37

class BackGroundColor(Enum):
    Black  = 40
    Blue   = 44
    Green  = 42
    Red    = 41
    Yellow = 43
    White  = 47


def myprint(desc,textColor=None,bgColor=None):
    if textColor is None and bgColor is None:
        sys.stdout.write(desc)
    else:
        color = ''
        if textColor is not None:
            color = str(textColor.value)
        if bgColor is not None:
            if color != '':
                color = color +';'
            color = color + str(bgColor.value)
        color = color + 'm'
        sys.stdout.write("\033[" + color + str(desc) + "\033[0m")


def showTable(columns, rows, colheadColor=None, colsColor=[]):
    """Display a table
    
    - columns: str[y] contains of all columns name
    - rows: str[x][y] table value
    - colheadColor: #TextColor# columns headColor color
    - colsColor: #TextColor# columns color, default is None  
    """
    try:
        # get columns width
        widths = []
        for item in columns:
            name = str(item)   
            widths.append(len(name))

        for rObj in rows:
            index = 0
            for item in rObj:
                if len(str(item)) > widths[index]:
                    widths[index] = len(str(item))
                index = index + 1
                if len(widths) <= index:
                    break
        
        boardstr = '-'
        for item in widths:
            for i in range(item + 2 + 1):
                boardstr = boardstr + '-'

        # print all columns name
        print(boardstr)
        index = 0
        for item in columns:
            item = item.center(widths[index] + 2)
            myprintNoEnter('|')
            myprint(item, colheadColor)
            index = index + 1
            if len(widths) <= index:
                break
        print('|')
        print(boardstr)

        # print value
        for rObj in rows:
            index = 0
            for index in range(len(columns)):
                if len(rObj) > index:
                    item = rObj[index]
                else:
                    item = ""
                color = None
                if len(colsColor) > index:
                    color = colsColor[index]

                item = (' ' + str(item)).ljust(widths[index] + 2)
                myprintNoEnter('|')
                myprint(item, color)
                index = index + 1
                if len(widths) <= index:
                    break
            print('|')
        print(boardstr)
        return True
    except:
        return False


# v = sys.version_info
# a = v[0]
# b = v[1]
# c = v[2]
# d = v[3]
# ff = myinputInt('input:',44)
# t = isInputYes(ff)
# f = myinput('tt')
# g = 0
cols = []
cols.append('SETTINGS')
cols.append('VALUE')
rows = []
rows.append(['xiaoming', 22, 181.2])
rows.append(['hong', 'static string pack(const char*format, ...)', 171])
rows.append(['guoqiang', 23, 190.5])
showTable(cols, rows, None,[TextColor.Green])



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   updateHelper.py
@Time    :   2018/12/20
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :   
'''
import os
import sys
import time
import shutil
import win32api

import aigpy.netHelper     as netHelper
import aigpy.pathHelper    as pathHelper
import aigpy.cmdHelper     as cmdHelper
import aigpy.systemHelper  as systemHelper
import aigpy.versionHelper as versionHelper
import aigpy.configHelper  as configHelper
import aigpy.zipHelper     as zipHelper

from aigpy.versionHelper import VersionFile

class updateTool(object):
    def __init__(self, in__file__, name, url, curver, mainFile, elseList, zipFile = None):
        self._file_  = in__file__
        self.tmpName = name + "-NewVersion"
        self.verName = name + '-verinfo.ini'
        self.curPath = systemHelper.getOwnPath(in__file__)
        self.tmpPath = self.curPath + '\\' + self.tmpName
        self.verPath = self.curPath + '\\' + self.verName

        self.curVer              = VersionFile()
        self.curVer.version      = curver
        self.curVer.mainFile     = mainFile
        self.curVer.elseFileList = elseList
        self.curVer.zipFile      = zipFile
        if zipFile is not None:
            self.curVer.isZip    = 1
        self.curVer.saveFile(self.verPath)

        self.netVer  = VersionFile()
        self.netUrl  = url
        return

    def _getNetVerinfo(self):
        tmpFile = self.curPath + '\\' + 'aigpy-tmp.ini'
        if not netHelper.downloadFile(self.netUrl + '//' + self.verName, tmpFile):
            return False
        return self.netVer.readFile(tmpFile)

    def isNeedUpdate(self):
        if self.curVer.version is None:
            return False
        if self.netVer.version is None:
            if self._getNetVerinfo() is False:
                return False
        return versionHelper.cmpVersion(self.curVer.version, self.netVer.version) < 0

    def _downloadFiles(self):
        check = pathHelper.remove(self.tmpPath)
        check = pathHelper.mkdirs(self.tmpPath)
        if self.netVer.mainFile is None:
            return False
        if self.netVer.isZip == 0:
            plist = []
            plist.append(self.netVer.mainFile)
            for item in self.netVer.elseFileList:
                plist.append(item)
            for item in plist:
                urlpath = self.netUrl + '//' + item
                topath  = self.tmpPath + '\\' + item
                if netHelper.downloadFile(urlpath, topath) is False:
                    return False
        else:
            urlpath = self.netUrl + '//' + self.netVer.zipFile
            topath = self.tmpPath + '\\' + self.netVer.zipFile
            if netHelper.downloadFile(urlpath, topath) is False:
                return False
            return zipHelper.unzip(topath, self.tmpPath)
        return True

    def go(self):
        bFlag = '-1'
        stri  = cmdHelper.findInArgv('AIGPYUPDATE=')
        if stri != None:
            bFlag = stri[len('AIGPYUPDATE='):]
        
        #Check New Version/Download New File To TmpDir/Start Tmp
        if bFlag == '-1':
            if self.isNeedUpdate() == False:
                return False
            if self._downloadFiles() == False:
                return False
            para = sys.argv
            para.append('AIGPYUPDATE=0')
            para.remove(para[0])
            win32api.ShellExecute(0, 'open', self.tmpPath + '\\' + self.netVer.mainFile, cmdHelper.converArgvToStr(para), self.tmpPath, 0)
            return True
        
        #Copy TmpFile To Cur Dir And Start 
        if bFlag == '0':
            time.sleep(3)
            
            ver     = VersionFile()
            tmpFile = self.curPath + '\\..\\' + 'aigpy-tmp.ini'
            ver.readFile(tmpFile)

            plist = []
            plist.append(ver.mainFile)
            for item in ver.elseFileList:
                plist.append(item)
            for item in plist:
                shutil.copyfile(self.curPath + '\\' + item, self.curPath + '\\..\\' + item)
            
            para = sys.argv
            para.remove(para[0])
            para.remove(stri)
            para.append('AIGPYUPDATE=2')
            win32api.ShellExecute(0, 'open', self.curPath + '\\..\\' + ver.mainFile, cmdHelper.converArgvToStr(para), self.curPath + '\\..\\', 1)
            return True
        
        return False
    



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

'''
@File    :   netHelper.py
@Time    :   2018/12/17
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :   
'''

import re
import os
import sys
import json
import socket
import requests
from aigpy.progressHelper import ProgressTool
from aigpy.convertHelper import convertStorageUnit
from aigpy.pathHelper import getFileName

def downloadString(url, timeouts=(3.05, 27)):
    try:
        re = requests.get(url, timeouts)
        return re.content
    except:
        return None 
    

def downloadJson(url, timeouts=(3.05, 27)):
    try:
        re = requests.get(url, timeouts)
        info = json.loads(re.content)
        return info
    except:
        return None

def getFileSize(url):
    if sys.version_info > (3, 0):
        from urllib.request import urlopen
    else:
        from urllib2 import urlopen

    try:
        response = urlopen(url)
        info     = response.info()
        dic      = dict(info)
        length   = dic['Content-Length']
        return int(length)
    except:
        return -1


def downloadFileByUrls(urlArray, fileName, stimeout=None, showprogress=False):
    if os.access(fileName, 0):
        os.remove(fileName)

    progress = None
    if showprogress:
        desc = getFileName(fileName)
        progress = ProgressTool(len(urlArray), 10, unit='', desc=desc)

    curcount = 1    
    for item in urlArray:
        ret = downloadFile(item, fileName, stimeout, False, append=True)
        if ret != True:
            return False
        if progress:
            progress.setCurCount(curcount)
            curcount+=1
    return True


def downloadFile(url, fileName, stimeout=None, showprogress=False, append=False):
    if sys.version_info > (3,0):
        from urllib.request import urlopen
    else:
        from urllib2 import urlopen
    
    try:
        if stimeout is None:
            response = urlopen(url)
        else:
            response = urlopen(url, timeout=stimeout)
        
        unit = 'mb'
        if convertStorageUnit(response.length, 'byte', unit) < 1:
            unit = 'kb'
        progress = None
        if showprogress:
            desc = getFileName(fileName)
            progress = ProgressTool(convertStorageUnit(response.length, 'byte', unit), 10, unit=unit, desc=desc)

        mode ='wb'
        if append:
            mode = 'ab'

        curcount  = 0
        chunksize = 16 * 1024
        with open(fileName, mode) as f:
            while True:
                chunk = response.read(chunksize)
                curcount += len(chunk)
                if progress:
                    progress.setCurCount(convertStorageUnit(curcount, 'byte', unit))
                if not chunk:
                    break
                f.write(chunk)
            return True
    except:
        return False


def getIpStatus(host, port, timeouts=1):
    socket.setdefaulttimeout(timeouts)
    flag = True
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((host, port))
        s.close()
    except:
        flag = False
    return flag


def getIP():
    text = requests.get("http://txt.go.sohu.com/ip/soip").text
    ip = re.findall(r'\d+.\d+.\d+.\d+', text)[0]
    return ip


def getResult(code=0,msg='',data=''):
    ret = {}
    ret['code']   = code
    ret['errmsg'] = msg
    ret['data']   = data
    return json.dumps(ret)




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   mathHelper.py
@Time    :   2019/02/27
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :   Math Common Function
'''


